/*-----------------------------------------------------------------------------
-- command.c
--
-- Copyright CERN, 2021
-- This source describes Open Hardware and is licensed under the
-- CERN-OHL-S v2
-- Source location: https://ohwr.org/project/pxie-ctl-comexpress-tst
-- designed by INCAA Computers
-----------------------------------------------------------------------------*/

/* Includes ------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <stdint.h>

#include "xparameters.h"
#include "xil_printf.h"
#include "xuartlite_l.h"

#include "platform.h"
#include "version.h"
#include "wd.h"
#include "gpio.h"
#include "at_engine.h"
#include "command.h"
#include "tFPGA_app.h"

#define ASCII_XON			0x11
#define ASCII_XOFF			0x13
#define ASCII_CR			0x0D // "\r"
#define ASCII_LF			0x0A // "\n"

#define CMD_STRING_BUF_SIZE	80
#define COMMANDS_MAX		50

#define CMD_ERR				0
#define CMD_OK				1

#define NO_TEST				NULL
#define NO_GET				NULL
#define NO_SET				NULL
#define NO_CONFIRM			NULL
#define NO_EXEC				NULL
	
static int 	menuGate;
static char commandBuffer[CMD_STRING_BUF_SIZE];
static char responseBuffer[CMD_STRING_BUF_SIZE];
static T_AT_storage  commands[COMMANDS_MAX];
	
typedef uint8_t ( *cmdTest_t )( uint32_t value);
typedef uint32_t ( *cmdGet_t )( void );
typedef void ( *cmdSet_t )( uint32_t value);
typedef uint8_t ( *cmdConfirm_t )( void );
typedef uint8_t ( *cmdExec_t )( void );

typedef struct
{
	cmdTest_t Test;
	cmdGet_t Get;
	cmdSet_t Set;
	cmdConfirm_t Confirm;
	cmdExec_t Exec;
}
cmdFunc_t;

/*-----------------------------------------------------------------------------            
        Private Function Definitions
 ------------------------------------------------------------------------------*/

static int kbhit (void)
{
	return (XUartLite_IsReceiveEmpty(XPAR_UARTLITE_0_BASEADDR) ? 0 : 1);
}

static char InputChar(void)
{
	int	i=0;
	char ch=0;

	while (i < 1) {
		ch = getchar();
		if (ch != ASCII_XON && ch != ASCII_XOFF) {
			i++;
		}
	}
	//xil_printf("%c", ch);
	return ch;
}

static int InputDecValue(void)
{
	int	i = 0;
	char ch, str[11];

	//xil_printf("* ");
	while (i < 10) {
		ch = InputChar();
		if (!isdigit(ch)) {
			if (i==0 && (ch == '-' || ch == '+')) {
				str[i++] = ch;
			}
			else {
				break;
			}
		}
		else {
			str[i++] = ch;
		}
		wd_Feed();
	}
	str[i] = '\0';
	//xil_printf("%s\n\r", str);
	return atoi(str);
}

static int InputKey(void)
{
	int	i = 0;
	char ch, str[6];

	//xil_printf("* ");
	while (i < 5) {
		ch = InputChar();
		if (!isdigit(ch)) {
			return -1;
		}
		else {
			str[i++] = ch;
		}
		wd_Feed();
	}
	str[5] = '\0';
	//xil_printf("%s\n\r", str);
	return atoi(str);
}

static bool ScanKey()
{
	int key;

	key = InputKey();
	if (key > 0 && key == 0xC0DE) {
		return true;
	}
	return false;
}

static uint8_t StringToDWord(uint32_t *value, char *str)
{
	int status;
	char *tmpStr;
	char prefix[3];
	
	tmpStr = str;
	if (strlen(tmpStr) > 2) {
		status = sscanf(tmpStr, "%2s", prefix);
		if ((strcmp(prefix, "0x") == 0) || (strcmp(prefix, "0X") == 0)) {
			tmpStr += 2;
			status = sscanf(tmpStr, "%lx", value);
		}
		else {
			status = sscanf(tmpStr, "%ld", value);
		}
	}
	else {
		status = sscanf(tmpStr, "%ld", value);
	}
	return status;
}

static void OutputDelay(void)
{
	sleep_ticks(25); // 25ms
}

void WriteChar(unsigned char data_out)
{
	fputc((int)data_out, stdout);
}

/*-----------------------------------------------------------------------------
        command handler function
 ------------------------------------------------------------------------------*/
void cmdFunctionsInit(cmdFunc_t *functions, cmdTest_t Test, cmdGet_t Get, cmdSet_t Set, cmdConfirm_t Confirm, cmdExec_t Exec)
{
	functions->Test = Test;
	functions->Get = Get;
	functions->Set = Set;
	functions->Confirm = Confirm;
	functions->Exec = Exec;
}

#define RESPONSE_ERROR		0
#define RESPONSE_OK			1
#define RESPONSE_STR		2

static char rspStr[64];

void cmdResponse(int type)
{
	switch (type) {
		case RESPONSE_ERROR:
			AT_rspStr("ERROR\n\r");
			break;
		case RESPONSE_OK:
			AT_rspStr("OK\n\r");
			break;
		case RESPONSE_STR:
			AT_rspStr(rspStr);
			AT_rspStr("\n\r");
			break;
		default:
			break;
	}
}

// Handler for decimal/hexadecimal parameter commands
static void cmdCommandHandler(char *buffer, uint8_t type, int argOff, uint32_t min, uint32_t max, cmdFunc_t *functions)
{
	uint32_t parameter;
	int rspType = RESPONSE_ERROR;
	
	switch(type) {
		case _AT_TEST: // <cmd>=?<parameter>
			if (functions->Test != NO_TEST) {
				StringToDWord(&parameter, buffer+argOff);
				//sscanf(buffer+argOff, "%d", &parameter);
				if (parameter >= min && parameter <= max) {
					if (functions->Test(parameter)) {
						rspType = RESPONSE_OK;
					}
				}
			}
			break;
		case _AT_GET: // <cmd>?
			if (functions->Get != NO_GET) {
				parameter = functions->Get();
				sprintf(rspStr, "0x%lx", parameter);
				rspType = RESPONSE_STR;
			}
			break;
		case _AT_SET: // <cmd>=<parameter>
			if (functions->Set != NO_SET) {
				StringToDWord(&parameter, buffer+argOff);
				//sscanf(buffer+argOff, "%d", &parameter);
				if (parameter >= min && parameter <= max) {
					functions->Set(parameter);
					rspType = RESPONSE_OK;
				}
			}
			break;
		case _AT_CONFIRM: // <cmd>!
			if (functions->Confirm != NO_CONFIRM) {
				if (functions->Confirm()) {
					rspType = RESPONSE_OK;
				}
			}
			break;
		case _AT_EXEC: // <cmd>
			if (functions->Exec != NO_EXEC) {
				if (functions->Exec()) {
					rspType = RESPONSE_OK;
				}
			}
			break;
		default: 
			break;
	}
	cmdResponse(rspType);
}

// Error handler
void cmdHandler_Error(char *buffer, uint8_t type, int argOff)
{
	if (strlen(buffer) > 1) {
		cmdResponse(RESPONSE_ERROR);
	}
}

// Handler for *IDN : IEEE Std 488.2-1992, identification query
void cmdHandler_IDN(char *buffer, uint8_t type, int argOff)
{
	int rspType = RESPONSE_ERROR;
	
	if (type == _AT_GET) { // *IDN?
		sprintf(rspStr, "%s,%s,0,%d", APP_MANUFACTURER, APP_INSTRUMENT, VERSION);
		rspType = RESPONSE_STR;
	}
	cmdResponse(rspType);
}

// Handler for *RST : IEEE Std 488.2-1992, full preset command
uint8_t cmdHandler_RST_EXEC(void)
{
	mb_reset();
	return CMD_OK;
}

void cmdHandler_RST(char *buffer, uint8_t type, int argOff)
{
	cmdFunc_t functions;
	//xil_printf(":RST\n\r");
	cmdFunctionsInit(&functions, NO_TEST, NO_GET, NO_SET, NO_CONFIRM, cmdHandler_RST_EXEC);
	cmdCommandHandler(buffer, type, argOff, 0, UINT32_MAX, &functions);
}

// Handler for :PXID : (MSB) pxi_trig7[2:0], pxi_trig[7:0] (LSB) Direction
void cmdHandler_PXID(char *buffer, uint8_t type, int argOff)
{
	cmdFunc_t functions;
	//xil_printf(":PXID\n\r");
	cmdFunctionsInit(&functions, NO_TEST, gpio_GetPxiDir, gpio_SetPxiDir, NO_CONFIRM, NO_EXEC);
	cmdCommandHandler(buffer, type, argOff, 0, GPIO_PXI, &functions);
}

// Handler for :PXI : (MSB) pxi_trig7[2:0], pxi_trig[7:0] (LSB) I/O
void cmdHandler_PXI(char *buffer, uint8_t type, int argOff)
{
	cmdFunc_t functions;
	//xil_printf(":PXI\n\r");
	cmdFunctionsInit(&functions, NO_TEST, gpio_GetPxi, gpio_SetPxi, NO_CONFIRM, NO_EXEC);
	cmdCommandHandler(buffer, type, argOff, 0, GPIO_PXI, &functions);
}

// Handler for :LBRD : pxi_lbr6 Direction
void cmdHandler_LBRD(char *buffer, uint8_t type, int argOff)
{
	cmdFunc_t functions;
	//xil_printf(":LBRD\n\r");
	cmdFunctionsInit(&functions, NO_TEST, gpio_GetLbrDir, gpio_SetLbrDir, NO_CONFIRM, NO_EXEC);
	cmdCommandHandler(buffer, type, argOff, 0, 1, &functions);
}

// Handler for :LBR : pxi_lbr6 I/O
void cmdHandler_LBR(char *buffer, uint8_t type, int argOff)
{
	cmdFunc_t functions;
	//xil_printf(":LBR\n\r");
	cmdFunctionsInit(&functions, NO_TEST, gpio_GetLbr, gpio_SetLbr, NO_CONFIRM, NO_EXEC);
	cmdCommandHandler(buffer, type, argOff, 0, 1, &functions);
}

// Handler for :GA : pxi_ga[4:0] Input
void cmdHandler_GA(char *buffer, uint8_t type, int argOff)
{
	cmdFunc_t functions;
	//xil_printf(":GA\n\r");
	cmdFunctionsInit(&functions, NO_TEST, gpio_GetPxiGa, NO_SET, NO_CONFIRM, NO_EXEC);
	cmdCommandHandler(buffer, type, argOff, 0, GPIO_PXI_GA, &functions);
}

// Handler for :STAR : pxi_star Input
void cmdHandler_STAR(char *buffer, uint8_t type, int argOff)
{
	cmdFunc_t functions;
	//xil_printf(":STAR\n\r");
	cmdFunctionsInit(&functions, NO_TEST, gpio_GetPxiStar, NO_SET, NO_CONFIRM, NO_EXEC);
	cmdCommandHandler(buffer, type, argOff, 0, 1, &functions);
}

// Handler for :STARR : pxi_star Input recorded at 100MS/s in a 32bit word
void cmdHandler_STARR(char *buffer, uint8_t type, int argOff)
{
	cmdFunc_t functions;
	//xil_printf(":STARR\n\r");
	cmdFunctionsInit(&functions, NO_TEST, gpio_GetPxiStarRec, NO_SET, NO_CONFIRM, NO_EXEC);
	cmdCommandHandler(buffer, type, argOff, 0, UINT32_MAX, &functions);
}

// Handler for :TRGD : front panel TRiGger Direction
void cmdHandler_TRGD_SET(uint32_t value)
{
	gpio_SetTrigDir(value);
	gpio_EnTrigTerm();
}

void cmdHandler_TRGD(char *buffer, uint8_t type, int argOff)
{
	cmdFunc_t functions;
	//xil_printf(":TRGD\n\r");l
	cmdFunctionsInit(&functions, NO_TEST, gpio_GetTrigDir, cmdHandler_TRGD_SET, NO_CONFIRM, NO_EXEC);
	cmdCommandHandler(buffer, type, argOff, 0, 1, &functions);
}

// Handler for :TRGT : front panel TRiGger Termination
void cmdHandler_TRGT(char *buffer, uint8_t type, int argOff)
{
	cmdFunc_t functions;
	//xil_printf(":TRGT\n\r");
	cmdFunctionsInit(&functions, NO_TEST, gpio_GetTrigTerm, gpio_SetTrigTerm, NO_CONFIRM, NO_EXEC);
	cmdCommandHandler(buffer, type, argOff, 0, 1, &functions);
}

// Handler for :TRG : front panel TRiGger I/O
void cmdHandler_TRG(char *buffer, uint8_t type, int argOff)
{
	cmdFunc_t functions;
	//xil_printf(":TRG\n\r");
	cmdFunctionsInit(&functions, NO_TEST, gpio_GetTrig, gpio_SetTrig, NO_CONFIRM, NO_EXEC);
	cmdCommandHandler(buffer, type, argOff, 0, 1, &functions);
}

// Handler for :GPO : FPGA_GPO[1:0] -> CEX_GPI[1:0]
void cmdHandler_GPO(char *buffer, uint8_t type, int argOff)
{
	cmdFunc_t functions;
	//xil_printf(":GPO\n\r");
	cmdFunctionsInit(&functions, NO_TEST, gpio_GetGpo, gpio_SetGpo, NO_CONFIRM, NO_EXEC);
	cmdCommandHandler(buffer, type, argOff, 0, GPIO_CEX_FPGA_GPO, &functions);
}

// Handler for :GPOE : Enable FPGA_GPO[1:0] -> CEX_GPI[1:0]
void cmdHandler_GPOE(char *buffer, uint8_t type, int argOff)
{
	cmdFunc_t functions;
	//xil_printf(":GPOE\n\r");
	cmdFunctionsInit(&functions, NO_TEST, gpio_GetGpoEnable, gpio_SetGpoEnable, NO_CONFIRM, NO_EXEC);
	cmdCommandHandler(buffer, type, argOff, 0, GPIO_CEX_FPGA_GPO, &functions);
}

// Handler for :GPI : FPGA_GPI[3:0] <- CEX_GPI[3:0]
void cmdHandler_GPI(char *buffer, uint8_t type, int argOff)
{
	cmdFunc_t functions;
	//xil_printf(":GPI\n\r");
	cmdFunctionsInit(&functions, NO_TEST, gpio_GetGpi, NO_SET, NO_CONFIRM, NO_EXEC);
	cmdCommandHandler(buffer, type, argOff, 0, GPIO_CEX_FPGA_GPI, &functions);
}

// Handler for :TEST1 : onboard TEST1 pin
void cmdHandler_TEST1(char *buffer, uint8_t type, int argOff)
{
	cmdFunc_t functions;
	//xil_printf(":TEST1\n\r");
	cmdFunctionsInit(&functions, NO_TEST, NO_GET, gpio_SetTest1, NO_CONFIRM, NO_EXEC);
	cmdCommandHandler(buffer, type, argOff, 0, 1, &functions);
}

// Handler for :TEST2 : onboard TEST2 pin
void cmdHandler_TEST2(char *buffer, uint8_t type, int argOff)
{
	cmdFunc_t functions;
	//xil_printf(":TEST2\n\r");
	cmdFunctionsInit(&functions, NO_TEST, NO_GET, gpio_SetTest2, NO_CONFIRM, NO_EXEC);
	cmdCommandHandler(buffer, type, argOff, 0, 1, &functions);
}


/*-----------------------------------------------------------------------------
        Public Function Definitions
 ------------------------------------------------------------------------------*/

void cmdInit(void)
{
	AT_initParser(WriteChar, cmdHandler_Error, commandBuffer, responseBuffer, CMD_STRING_BUF_SIZE, commands, COMMANDS_MAX);
	
	AT_saveHandler("*IDN", cmdHandler_IDN);			// IEEE Std 488.2-1992, identification query
	AT_saveHandler("*RST", cmdHandler_RST);			// IEEE Std 488.2-1992, full reset command

	AT_saveHandler(":PXID", cmdHandler_PXID);		// 	pxi_trig7_t[2:0], pxi_trig_t[7:0]
	AT_saveHandler(":PXI", cmdHandler_PXI);			//	pxi_trig7_i/o[2:0], pxi_trig_i/0[7:0]

	AT_saveHandler(":LBRD", cmdHandler_LBRD);		// 	pxi_lbr6_t
	AT_saveHandler(":LBR", cmdHandler_LBR);			//	pxi_lbr6_i/o

	AT_saveHandler(":GA", cmdHandler_GA);			//	pxi_ga[4:0]
	AT_saveHandler(":STAR", cmdHandler_STAR);		//	pxi_star
	AT_saveHandler(":STARR", cmdHandler_STARR);		//	pxi_star recorded

	AT_saveHandler(":TRGD", cmdHandler_TRGD);		//	trig_oe
	AT_saveHandler(":TRGT", cmdHandler_TRGT);		//	trig_tsel
	AT_saveHandler(":TRG", cmdHandler_TRG);			//	trig_in/out

	AT_saveHandler(":GPO", cmdHandler_GPO);			//	fpga_gpo[1:0]
	AT_saveHandler(":GPOE", cmdHandler_GPOE);		//	fpga_gpo_sel_N
	AT_saveHandler(":GPI", cmdHandler_GPI);			//	fpga_gpi[3:0]

	AT_saveHandler(":TEST1", cmdHandler_TEST1);		//	test[1]
	AT_saveHandler(":TEST2", cmdHandler_TEST2);		//	test[2]
}

void cmdParser(void)
{
	char ch;
	
	while (kbhit()) {
		ch = getchar();
		//xil_printf("%c", ch);
		//xil_printf("<0x%x>", ch);
		switch (ch) {
			case ASCII_XON:
			case ASCII_XOFF:
				// Ignore XON and XOFF
				break;
			case '#': // Close menu using "#"
				menuGate = 0;		
				xil_printf("* OK\n\r");
				break;
			case '!': // Open menu using "!!!"
				menuGate++;
				if (menuGate == 3) {
					xil_printf("* OK\n\r");
				}
				else if (menuGate > 3) {
					menuGate = 0;							
				}
				break;
			default:
				if (menuGate == 3) {
					// Menu open
					switch (ch) {
						case ' ': // menu
						case '?': // help
							xil_printf("* R \t\t Reboot.\n\r"); OutputDelay();
							xil_printf("* v \t\t Version information.\n\r"); OutputDelay();
							break;
						case 'v': // version
							xil_printf(APP_SPLASH, VERSION); OutputDelay();
							break;
						case 'F': // get firmware version
							xil_printf("* %d\n\r", VERSION); OutputDelay();
							break;
							
						case 'R': // Reboot
							if (ScanKey()) {
								mb_reset();
							}
							break;
							
						default :
							menuGate = 0;
							break;
					}	
				}
				else {
					// Not in the menu
					menuGate = 0;
					
					// Handle the commands
					switch (ch) {
						case ASCII_LF:
							break;
						case ASCII_CR:
							AT_cmdChar(ASCII_CR);
							AT_cmdChar(ASCII_LF);
							AT_process(); // also allow higher clearance level commands
							AT_reply();
							break;
						default:
							AT_cmdChar(ch);
							break;
					}
				}												
				break;						
		}
	}
}

/****END OF FILE****/
