/*-----------------------------------------------------------------------------
-- wd.c
--
-- Copyright CERN, 2021
-- This source describes Open Hardware and is licensed under the
-- CERN-OHL-S v2
-- Source location: https://ohwr.org/project/pxie-ctl-comexpress-tst
-- designed by INCAA Computers
-----------------------------------------------------------------------------*/

#include "xparameters.h"
#include "xstatus.h"
#include "xwdttb_l.h"

#include "wd.h"

/*
 * When the AXI Timebase Watchdog Timer asserts wdt_reset and this
 * should reset the Processor System Reset this will case a problem
 * which is reported as:
 * "incorrect configuration: xps_ethernetlite drivers not present?"
 *
 * This is "normal" with a processor reset. When Microblaze is restarted,
 * it starts to execute from 0x00000000. If the application start and run
 * correctly depends mainly by role of initialized data. When working
 * without a bootloader, and loading the applicatio with SDK.
 * When the application is loaded, section .data is loaded by SDK, and
 * initialized variable have correct values. When the Microblaze restarts
 * the application initialized variable have different values in respect
 * to the initial values, and Microblaze run time doesn't reinit .data
 * variable, then this segment must be restored the initial value,
 * otherwise the application fails to start. Refer to AR# 30878.
 * When using a bootloader and with an internal auto reset situation is
 * more complicated because you need to disable interrupt and restore
 * Microblaze registers.
 * To overcome this all the wdt_reset could trigger a reconfiguration of
 * the FPGA.
 */

#define WDTTB_BASEADDR XPAR_WDTTB_0_BASEADDR

XStatus wd_Init(void)
{
#ifdef tFPGA_RELEASE
	/*
	 * Set the registers to enable the watchdog timer, both enable bits
	 * in TCSR0 and TCSR1 need to be set to enable it.
	 * Clear the bit that indicates the reason for the last
	 * system reset, WRS and the WDS bit, if set, by writing
	 * 1's to TCSR0
	 */
	XWdtTb_WriteReg(WDTTB_BASEADDR, XWT_TWCSR0_OFFSET, (XWT_CSR0_WRS_MASK | XWT_CSR0_WDS_MASK | XWT_CSR0_EWDT1_MASK));
	XWdtTb_WriteReg(WDTTB_BASEADDR, XWT_TWCSR1_OFFSET, XWT_CSRX_EWDT2_MASK);
#else
#warning ">>>>> WATCHDOG DISABLED <<<<<"
	xil_printf(">>>>> WATCHDOG DISABLED <<<<<\n\r");
#endif
	return XST_SUCCESS;
}

void wd_Feed(void)
{
	XWdtTb_WriteReg(WDTTB_BASEADDR, XWT_TWCSR0_OFFSET, (XWT_CSR0_WRS_MASK | XWT_CSR0_WDS_MASK | XWT_CSR0_EWDT1_MASK));
}

u32 wd_TimerValue(void)
{
	return XWdtTb_ReadReg(WDTTB_BASEADDR, XWT_TBR_OFFSET);
}
