#!   /usr/bin/env   python
#    coding: utf8

# Copyright CERN, 2021-2022
# This source describes Open Hardware and is licensed under the
# CERN-OHL-W-2.0+
# Source location: https://ohwr.org/project/pxie-ctl-comexpress-tst
# designed by INCAA Computers
#------------------------------------------------------------------------
# Changes
# 2022-12-15 INCAA Computers
#   Added label to _parameters
#   Added _get_baseaddress() to find the base address
#   Added _get_gpiochip_label()
#   Added baseaddr()
#------------------------------------------------------------------------

import sys
import os
import subprocess
import logging
logging.getLogger("gpio").setLevel(logging.INFO)

from Item import *

"""
Simple interface to COMe GPIO interface
"""

class COMe_GPIO(Item):

    _parameters = {'addr':['Base address', 'Base GPIO sysfs address', 0, int],
                  'label':['gpiochip device label', 'Label of the sysfs gpiochip', "gpio-kempld", str],
                  'gpi_offset':['GPI address offset', 'Offset GPI sysfs address', 0, int],
                  'gpo_offset':['GPO address offset', 'Offset GPO sysfs address', 4, int]}

    IN, OUT = 'in', 'out'

    def __init__(self, *args, **kwargs):
        """The initializer doesn't connect to the device."""
        Item.__init__(self, *args, **kwargs)
        if (self.addr == 0):
            self.addr = self._get_baseaddress()
        self.gpo_base = self.addr + self.gpo_offset
        self.gpi_base = self.addr + self.gpi_offset

    def _shell(self, sysfs_cmd):
        try:
            df = subprocess.check_output(sysfs_cmd, shell=True, stderr=subprocess.STDOUT)
            output = df.rstrip()
            status = 0
        except subprocess.CalledProcessError as e:
            output = e.output.rstrip()
            status = e.returncode
        return output, status

    def _get_baseaddress(self):
        cmd_cd = "cd /sys/class/gpio"
        cmd_ls = "ls -d gpiochip*" 
        sysfs_shell_cmd = cmd_cd + "; " + cmd_ls
        output, status = self._shell(sysfs_shell_cmd)
        base = 0
        if status == 0:
            output_lines = output.split('\n')
            for line in output_lines:
                line = line.lstrip()
                line = line.rstrip()
                sbase = line.replace("gpiochip", "")
                ibase = int(sbase)
                label = self._get_gpiochip_label(ibase)
                if base == 0 and label == self.label:
                    base = ibase
        return base
            
    def _get_gpiochip_label(self, base):
        sysfs_cmd = "cat /sys/class/gpio/gpiochip%d/label" % (base)
        output, status = self._shell(sysfs_cmd)
        return (output)
    
    def _unexport(self, chippin):
        sysfs_cmd_unexport = "echo %d > /sys/class/gpio/unexport" % (chippin)
        output, status = self._shell(sysfs_cmd_unexport)
        if status != 0:
            sysfs_cmd_export = "echo %d > /sys/class/gpio/export" % (chippin)
            output, status = self._shell(sysfs_cmd_export)
        if status == 0:
            output, status = self._shell(sysfs_cmd_unexport)
        return output    

    def _export(self, chippin):
        sysfs_cmd_export = "echo %d > /sys/class/gpio/export" % (chippin)
        output, status = self._shell(sysfs_cmd_export)
        if status != 0:
            sysfs_cmd_unexport = "echo %d > /sys/class/gpio/unexport" % (chippin)
            output, status = self._shell(sysfs_cmd_export)
        if status == 0:
            output, status = self._shell(sysfs_cmd_export)
        return output    

    def _set_direction(self, pin, direction):
        sysfs_cmd = "echo %s > /sys/class/gpio/GPIO%d/direction" % (direction, pin)
        self._shell(sysfs_cmd)

    def _set_value(self, pin, value):
        sysfs_cmd = "echo %d > /sys/class/gpio/GPIO%d/value" % (value, pin)
        self._shell(sysfs_cmd)

    def _get_value(self, pin):
        sysfs_cmd = "cat /sys/class/gpio/GPIO%d/value" % (pin)
        output, status = self._shell(sysfs_cmd)
        return int(output)

    def _setupGPI(self, pin):
        if (pin < 0 | pin > 3):
            return
        gpi_chippin = int(pin) + self.gpi_base
        gpi_pin = int(pin) + self.gpi_offset
        self._export(gpi_chippin)
        self._set_direction(gpi_pin, self.IN)

    def _setupGPO(self, pin):
        if (pin < 0 | pin > 3):
            return
        gpo_chippin = int(pin) + self.gpo_base
        gpo_pin = int(pin) + self.gpo_offset
        self._export(gpo_chippin)
        self._set_direction(gpo_pin, self.OUT)

    def _cleanupGPI(self, pin):
        if (pin < 0 | pin > 3):
            return
        gpi_chippin = int(pin) + self.gpi_base
        self._unexport(gpi_chippin)

    def _cleanupGPO(self, pin):
        if (pin < 0 | pin > 3):
            return
        gpo_chippin = int(pin) + self.gpo_base
        self._unexport(gpo_chippin)

    def setup(self):
        for i in range(0, 4):
            self._setupGPO(i)
            self._setupGPI(i)
    
    def cleanup(self):
        for i in range(0, 4):
            self._cleanupGPO(i)
            self._cleanupGPI(i)

    def gpo(self, pin, value):
        gpio_pin = int(pin) + self.gpo_offset
        self._set_value(gpio_pin, value)

    def gpi(self, pin):
        gpio_pin = int(pin) + self.gpi_offset
        return int(self._get_value(gpio_pin))

    def baseaddr(self):
        return self.addr
            
if __name__ == "__main__":
    main()
    
        

