#!   /usr/bin/env   python
#    coding: utf8

# Copyright CERN, 2021-2022
# This source describes Open Hardware and is licensed under the
# CERN-OHL-W-2.0+
# Source location: https://ohwr.org/project/pxie-ctl-comexpress-tst
# designed by INCAA Computers

import serial

from Item import *

"""
Simple interface to COMe serial interface
"""

class COMe_Serial(Item):

    # we assume 115200 8N1
    _parameters = {'device':['Serial device', 'Serial device used to communicate', "/dev/ttyUSB0", 'file'],
                  'bauds':['Bauds', 'Speed of the communication', 115200, int],
                  'to':['Timeout', 'Timeout during read operations',  1, int]}


    def __init__(self, *args, **kwargs):
        """The initializer doesn't connect to the device."""
        Item.__init__(self, *args, **kwargs)
        self.connected = False

        # initialize
    def connect(self):
        """ Connect to the device"""
        self.device = serial.Serial(port = self.device,
                             baudrate = self.bauds,
                             bytesize = serial.EIGHTBITS,
                             parity = serial.PARITY_NONE,
                             stopbits = serial.STOPBITS_ONE,
                             timeout = self.to,
                             write_timeout = 1,
                             xonxoff = False,
                             rtscts = False,
                             dsrdtr = False)
        if self.device.is_open:
            self.close()
        self.device.open()
        self.device.flush()
        self.connected = True
        
    def write(self, str):
        if self.connected:
            self.device.write(str)
    
    def read(self, len):
        self.res = ""
        if self.connected:
            self.res = self.device.read(len)
        return self.res

    def writeline(self, str):
        if self.connected:
            self.device.write(str)
            self.device.write("\r\n")
    
    def readline(self):
        self.res = ""
        if self.connected:
            while self.res.find('\n') == -1:
                ch = self.device.read()
                if len(ch) == 0:
                    break
                self.res += ch
        return self.res
    
    def close(self):
        """ Close the connection to the device"""
        self.device.close()
        self.connected = False
        
    def send(self, what):
        if len(what) == 0: 
            return
        if type(what) is str:
            what = (what, )
        if not self.connected:
            self.connect()
        return self.writeline("%s" % what)

    def response(self):
        res = self.readline()
        res = res.lstrip()
        res = res.rstrip()
        return res

    def int_response(self):
        res = self.response()
        res = res.upper()
        if res == "" or res == "ERROR":
            ret_int = -1
        else:
            if res == "OK":
                ret_int = 0
            else:
                ret_int = int(res, base=16)
        return ret_int

    def command(self, what):
        self.send(what)
        return self.int_response()

    # id: IEEE Std 488.2-1992, identification query
    def id(self):
        self.send("*IDN?")
        return self.response()

    # reset: IEEE Std 488.2-1992, full reset command
    def reset(self):
        self.send("*RST")
    
if __name__ == "__main__":
    main()
    
        

