#!   /usr/bin/env   python
#    coding: utf8

# Copyright CERN, 2021-2022
# This source describes Open Hardware and is licensed under the
# CERN-OHL-W-2.0+
# Source location: https://ohwr.org/project/pxie-ctl-comexpress-tst
# designed by INCAA Computers
#------------------------------------------------------------------------
# Changes
# 2022-11-15 INCAA Computers
#   Added documentation
#   Changed COMe_module_settings to include the bios_version
#   Added com0_device and com1_device within COMe_module_settings
#   Added the COMe-bTL6 BIOS version BTL6R901 within COMe_module_settings
#   Added COMe_bios_version()
#   Added COMe_module_com0_device() and COMe_module_com1_device()
# 2022-12-15 INCAA Computers
#   Changed documentation
#   Removed gpio_base_addr (base addr of the gpio is now set dynamically)
#   Removed COMe_module_gpio_base_addr
#   Added gpiochip_label within COMe_module_settings
#   Added COMe_module_gpiochip_label
#------------------------------------------------------------------------

import sys
import time
import os
import subprocess

"""
PXIeCOMe PTS common functions and settings

The setting which support the possibility to use serveral different COMe modules
are available using COMe_module_settings.
This COMe_module_settings consists of a heirarchy using the board_name and bios_vesion.

board_name
    This is the value which is returned by the shell command 'cat /sys/class/dmi/id/board_name' 

bios_vesion
    This is the value which is returned by the shell command 'cat /sys/class/dmi/id/bios_version'

gpiochip_label
    The device label of the gpio chip which provides access to the COMe module GPI[0:3] and
    GPO[0:3] pins.
    Use the shell command 'll /sys/class/gpio' or 'ls -l /sys/class/gpio' to see which gpio
    chips are avaible.
    For Kontron boards the one using the device kempld-gpio should be used. The number XXX
    within gpiochipXXX is the gpio base address, which has to be found dynamically based on
    the label of the gpio chip.
    The shell command 'cat /sys/class/gpio/gpiochip1016/label' will return the value for
    gpiochip_label. For Kontron boards this will be 'gpio-kempld'.

i2c_port
    The port number for the i2c port which provides access to the COMe module I2C I2C_CK and
    I2C_DAT pins.
    Use the shell command 'i2cdetect -l' to show the list of ports.
    For Kontron boards the i2c adapter 'i2c-kempld' should be used and the i2c-x shows with x
    the port number. So when 'i2c-kempld' is connected to i2c-9, i2c_port should be 9.

smbus_port
    The port number for the SMBus port which provides access to the COMe module SMBus SMB_CK
    and SMB_DAT pins.
    Use the shell command 'i2cdetect -l' to show the list of ports.
    For Kontron boards the SMBus adapter 'SMBus CMI adapter cmi' should be used and the i2c-x
    shows with x the port number. So when 'SMBus CMI adapter cmi' is connected to i2c-7,
    smbus_port should be 7.

net_device
    The network device which provides access to the COMe module GBE0 interface pins.
    Use the shell command 'll /sys/class/net' should show which device is not virtual.

com0_device
    The serial ports (usually named COM0 within the BIOS) which provide access to the COMe
    module serial port SER0_RX and SER0_TX.
    Use the shell command 'dmesg | grep ttyS' to show which uart com port uses which ttyS device.
    For Kontron boards SER0/COM0 is connected to dw-apb-uart.0

com1_device
    The serial ports (usually named COM1 within the BIOS) which provide access to the COMe
    module serial port SER1_RX and SER1_TX.
    Use the shell command 'dmesg | grep ttyS' to show which uart com port uses which ttyS device.
    For Kontron boards SER1/COM1 is connected to dw-apb-uart.1

"""

COMe_module_settings = {
    'COMe-bCL6': { # board_name
        'BCL6R113': { # bios_version
            'gpiochip_label' : 'gpio-kempld',
            'i2c_port' : 9,
            'smbus_port' : 7,
            'net_device' : 'eno1',
            'com0_device' : '/dev/ttyS0',
            'com1_device' : '/dev/ttyS1' 
            },
        },
    'COMe-bTL6': { # board_name
        'BTL6R110': { # bios_version
            'gpiochip_label' : 'gpio-kempld',
            'i2c_port' : 15,
            'smbus_port' : 13,
            'net_device' : 'enp35s0',
            'com0_device' : '/dev/ttyS0',
            'com1_device' : '/dev/ttyS1' 
            },
        'BTL6R901': { # bios_version
            'gpiochip_label' : 'gpio-kempld',
            'i2c_port' : 15,
            'smbus_port' : 13,
            'net_device' : 'enp35s0',
            'com0_device' : '/dev/ttyS1',
            'com1_device' : '/dev/ttyS2' 
            }
        }
    }


def shell_cat_file(f):
    if os.path.exists(f):
        sysfs_cmd = "cat " + f
        df = subprocess.check_output(sysfs_cmd, shell=True)
        return df.rstrip()
    return "None"

def current_time():
    return round(time.time())

def print_error():
    print " [Error]"

def print_ok():
    print " [Ok]"

def COMe_module_attrib(attrib):
    sysfs_file = '/sys/class/dmi/id/%s' % attrib
    return shell_cat_file(sysfs_file)

def COMe_module_name():
    return COMe_module_attrib('board_name')

def COMe_bios_version():
    return COMe_module_attrib('bios_version')

def COMe_module_gpiochip_label():
    return COMe_module_settings[COMe_module_name()][COMe_bios_version()]['gpiochip_label']

def COMe_module_i2c_port():
    return COMe_module_settings[COMe_module_name()][COMe_bios_version()]['i2c_port']

def COMe_module_smbus_port():
    return COMe_module_settings[COMe_module_name()][COMe_bios_version()]['smbus_port']
    
def COMe_module_net_device():
    return COMe_module_settings[COMe_module_name()][COMe_bios_version()]['net_device']

def COMe_module_com0_device():
    return COMe_module_settings[COMe_module_name()][COMe_bios_version()]['com0_device']

def COMe_module_com1_device():
    return COMe_module_settings[COMe_module_name()][COMe_bios_version()]['com1_device']

