#!   /usr/bin/env   python
#    coding: utf8

# Copyright CERN, 2021-2022
# This source describes Open Hardware and is licensed under the
# CERN-OHL-W-2.0+
# Source location: https://ohwr.org/project/pxie-ctl-comexpress-tst
# designed by INCAA Computers

import sys
import time
import os
import subprocess

# Add common modules and libraries location to path
sys.path.append('../../../')

from ptsexcept import *
from ptspxiecome import *

"""
test01: Check SATA/mSATA and LEDs

Since the PTS must boot from the PXIeCOMe mSATA SSD, the fact that the
system is running already indicated the SATA1 interface is working.
This test logs several ATA device aspects and tests the main front panel LEDS
"""

def ata_port_link_sysfs(port):
    sysfs_path = '/sys/class/ata_port/ata%d/device/link1/' % (port)
    return sysfs_path

def ata_port_host_sysfs(port, host):
    sysfs_path = '/sys/class/ata_port/ata%d/device/host%d/' % (port, host)
    return sysfs_path

def ata_port_host_target_sysfs(port, host):
    sysfs_path = ata_port_host_sysfs(port, host) + 'target%d:0:0/%d:0:0:0/' % (host, host)
    return sysfs_path

def ata_port_link_device_sysfs(port):
    sysfs_path = ata_port_link_sysfs(port) + 'dev1.0/ata_device/dev1.0/' 
    return sysfs_path

def ata_link_speed(link):
    sysfs_file = ata_port_link_sysfs(link) + 'ata_link/link%d/sata_spd' % (link)
    return shell_cat_file(sysfs_file)

def ata_link_device_class(link):
    sysfs_file = ata_port_link_device_sysfs(link) + 'class'
    return shell_cat_file(sysfs_file)

def ata_link_device_id(link):
    sysfs_file = ata_port_link_device_sysfs(link) + 'id'
    return shell_cat_file(sysfs_file)

def ata_link_device_xfer_mode(link):
    sysfs_file = ata_port_link_device_sysfs(link) + 'xfer_mode'
    return shell_cat_file(sysfs_file)

def ata_link_device_model(link):
    for host in range(1, 5):
        sysfs_file = ata_port_host_target_sysfs(link, host) + 'model'
        model = shell_cat_file(sysfs_file)
        if model != "None":
            return model
    return "None"

def ata_link_device_wwid(link):
    for host in range(1, 5):
        sysfs_file = ata_port_host_target_sysfs(link, host) + 'wwid'
        model = shell_cat_file(sysfs_file)
        if model != "None":
            return model
    return "None"

def ata_activity():
    string_val = "." * 65536
    tmp_file = "./ata.tmp"
    if os.path.exists(tmp_file):
        os.remove(tmp_file)
    f = open(tmp_file, "a")
    curtime = current_time()
    timeout = 4
    count = timeout
    while 1:
        f.write(string_val)
        count = timeout - (current_time() - curtime)
        if count < 0:
            count = 0
        print "%d\r" % (count),
        if (current_time() - curtime) > timeout:
            break
    f.close()
    os.remove(tmp_file)
    print ""

tmp_stdout = sys.stdout
tmp_stdin = sys.stdin

def stdio_redirect_start():
    tmp_stdout = sys.stdout
    sys.stdout = sys.__stdout__
    tmp_stdin = sys.stdin
    sys.stdin = sys.__stdin__

def stdio_redirect_end():
    sys.stdout = tmp_stdout
    sys.stdin = tmp_stdin


def main (default_directory='.'):

    print "Test01 (ATA) Start"
    init_test_time = time.time()
    ret_error = None
    abort = None

    stdio_redirect_start()
    print ""
    ask = "";
    while ((ask != "Y") and (ask != "N") and (ask != "A")):
        print "-------------------------------------------------------------"
        ask = raw_input("Is the front panel LED (PWR) switched on? [Y(es),N(o),A(bort)]")
        ask = ask.upper()
        print " "
    stdio_redirect_end()
    if (ask == "A"):
        abort = "Test aborted"
    else:
        print 'LED (PWR)',
        if (ask == "N"):
            ret_error = 'LED (PWR) not functioning.'
            print_error()
        if (ask == "Y"):
            print_ok()
    if abort:
        raise PtsCritical(abort)

    stdio_redirect_start()
    ask = "";
    while ((ask != "Y") and (ask != "A")) :
        print "-------------------------------------------------------------"
        print "Observe the front panel LED (DRIVE)"
        ask = raw_input("Ready to start the observation? [Y(es),A(bort)]")
        ask = ask.upper()
        print " "
    stdio_redirect_end()
    if (ask == "A"):
        abort = "Test aborted"
    if abort:
        raise PtsCritical(abort)

    stdio_redirect_start()
    ata_activity()
    ask = "";
    while ((ask != "Y") and (ask != "N") and (ask != "A")):
        ask = raw_input("Was the front panel LED (DRIVE) switched on or blinking? [Y(es),N(o),A(bort)]")
        ask = ask.upper()
        print " "
    stdio_redirect_end()
    if (ask == "A"):
        abort = "Test aborted"
    else:
        print 'LED (DRIVE)',
        if (ask == "N"):
            ret_error = 'LED (DRIVE) not functioning.'
            print_error()
        if (ask == "Y"):
            print_ok()
    if abort:
        raise PtsCritical(abort)
    
    # list ATA related information
    print "Class: %s\n" % (ata_link_device_class(1))
    print "Speed: %s\n" % (ata_link_speed(1))
    print "Xfer modes: %s\n" % (ata_link_device_xfer_mode(1))
    print "Model: %s\n" % (ata_link_device_model(1))
    print "WWID: %s\n" % (ata_link_device_wwid(1))
    print "ID:\n%s\n" % (ata_link_device_id(1))

    end_test_time = time.time()
    print "End of Test01 (ATA)"
    print "RESULT: [{}]".format("FAIL" if ret_error else "OK")
    print 'Test01 elapsed time: {:.2f} seconds'.format(end_test_time-init_test_time)
    if ret_error:
        raise PtsError(ret_error)
    

if __name__ == '__main__' :
    main()
