#!   /usr/bin/env   python
#    coding: utf8

# Copyright CERN, 2021-2022
# This source describes Open Hardware and is licensed under the
# CERN-OHL-W-2.0+
# Source location: https://ohwr.org/project/pxie-ctl-comexpress-tst
# designed by INCAA Computers

import sys
import time
import os
import subprocess

# Add common modules and libraries location to path
sys.path.append('../../../')

from ptsexcept import *
from ptspxiecome import *

"""
test04: Check USB

Check the speed of the USB ports. A speed of 5000 Mbps or higher is the indication
for the use of the Super Speed TX and RX pairs. A speed of 480 Mbps (High Speed),
12 Mbps (Full Speed) or 1.5 Mbps (Low Speed) is the indication for the use of
the D+/D- signal pair.
"""

setup_image = [
"+-----------PXIeCOMe--------------+--PXCT--+",
"|           LAN     USB           |        | Only CHANGE these interfaces:",
"| 10/100 +------+ +-+ +-+         |        | ",
"|  /1000 |o     | | | | | o PWR   | o      | -USB6 to USB memory stick  <-----",
"|        |o     | |6| |5| o DRIVE | o STAT | -USB5 to USB memory stick  <--  |",
"|   ACT/ +------+ +-+ +-+         |        |                              |  |",
"|   LINK          +-+ +-+         |        |                              |  |",
"|                 | | | | USB     | /--\   |                              |  |",
"|          /--\   |4| |3|         | |  |   |                              |  |",
"|   TRIG   |  |   +-+ +-+         | \--/   |                              |  |",
"|          \--/   +-+ +-+         |   TRIG |                              |  |",
"|                 | | | | USB (SS)|        | -USB2 to mouse    <----------|---",
"|         +---\   |2| |1|         |  /---+ | -USB1 to keyboard <-----------",
"|         |   |   +-+ +-+         |  |   | | ",
"|    COM  |   |       +-+         |  |   | | So swap the connections between",
"|         |   |       | |         |  |   | | USB1 and USB5, and",
"|         +---/       | | DP      |  \---+ | USB2 and USB6",
"|                     | |         |   COM  | And leave all other interfaces",
"|                     +-/         |        | connected as they were.",
"+------------------------+-Slot 1-+-Slot 2-+"
]

def usb_port_cat_sysfs(port, attrib):
    for bus in range(1, 5):
        sysfs_file = '/sys/bus/usb/devices/%d-%d/%s' % (bus, port, attrib)
        if os.path.exists(sysfs_file):
            sysfs_cmd = "cat " + sysfs_file
            df = subprocess.check_output(sysfs_cmd, shell=True)
            return df.rstrip()
    return "None"

def usb_port_speed(port):
    return usb_port_cat_sysfs(port, 'speed')

def usb_port_version(port):
    return usb_port_cat_sysfs(port, 'version')

def usb_port_product(port):
    return usb_port_cat_sysfs(port, 'product')

tmp_stdout = sys.stdout
tmp_stdin = sys.stdin

def stdio_redirect_start():
    tmp_stdout = sys.stdout
    sys.stdout = sys.__stdout__
    tmp_stdin = sys.stdin
    sys.stdin = sys.__stdin__

def stdio_redirect_end():
    sys.stdout = tmp_stdout
    sys.stdin = tmp_stdin


def main (default_directory='.'):

    print "Test04 (USB) Start"
    init_test_time = time.time()
    ret_error = None
    abort = None

    stdio_redirect_start()
    ask = "";
    while ((ask != "Y") and (ask != "N") and (ask != "A")):
        print "-------------------------------------------------------------"
        print "Are the two USB3.0 memory sticks installed in the two blue USB (SS) ports?"
        ask = raw_input("[Y(es),N(o),A(bort)]")
        ask = ask.upper()
        print " "
    stdio_redirect_end()
    if (ask == "A"):
        abort = "Test aborted"
    else:
        print 'USB3.0 memory sticks installed',
        if (ask == "N"):
            ret_error = 'Incorrect setup.'
            print_error()
        if (ask == "Y"):
            print_ok()
    if abort:
        raise PtsCritical(abort)
    
    # Check USB1 and USB2 for USB3.x devices
    for port in range(1, 3):
        print "USB%d product: %s" % (port, usb_port_product(port))
        print "USB%d version: %s" % (port, usb_port_version(port))
        speed = usb_port_speed(port)
        if speed == "None":
            speed = "0"
        print "USB%d speed: %s Mbps" % (port, speed),
        i_speed = int(round(float(speed)))
        if i_speed < 5000:
            ret_error = 'USB3.0 error.'
            print_error()
        else:
            print_ok()
        print ""

    stdio_redirect_start()
    print ("")
    for i in range(len(setup_image)):
        print (" %s") % setup_image[i]
    print ("")
    ask = "";
    while ((ask != "Y") and (ask != "N") and (ask != "A")):
        print "-------------------------------------------------------------"
        ask = raw_input("Has the test setup been changed as shown above? [Y(es),N(o),A(bort)]")
        ask = ask.upper()
        print " "
    stdio_redirect_end()
    if (ask == "A"):
        abort = "Test aborted"
    else:
        print 'USB test setup changed',
        if (ask == "N"):
            ret_error = 'Incorrect setup.'
            print_error()
        if (ask == "Y"):
            print_ok()
    if abort:
        raise PtsCritical(abort)
    
    # Check USB1 to USB6 for USB2.x or USB1.x devices
    for port in range(1, 7):
        print "USB%d product: %s" % (port, usb_port_product(port))
        print "USB%d version: %s" % (port, usb_port_version(port))
        speed = usb_port_speed(port)
        if speed == "None":
            speed = "0"
        print "USB%d speed: %s Mbps" % (port, speed),
        i_speed = int(round(float(speed)))
        if i_speed > 480 or i_speed <= 0:
            ret_error = 'USB2.0 error.'
            print_error()
        else:
            print_ok()
        print ""

    end_test_time = time.time()
    print "End of Test04 (USB)"
    print "RESULT: [{}]".format("FAIL" if ret_error else "OK")
    print 'Test04 elapsed time: {:.2f} seconds'.format(end_test_time-init_test_time)
    if ret_error:
        raise PtsError(ret_error)
    

if __name__ == '__main__' :
    main()
