#!   /usr/bin/env   python
#    coding: utf8

# Copyright CERN, 2021-2022
# This source describes Open Hardware and is licensed under the
# CERN-OHL-W-2.0+
# Source location: https://ohwr.org/project/pxie-ctl-comexpress-tst
# designed by INCAA Computers

import sys
import time
import os
import subprocess

# Add common modules and libraries location to path
sys.path.append('../../../')

from ptsexcept import *
from ptspxiecome import *

"""
test05: Check JTAG 

Check JTAG by programming the SPI flash of the FPGA. And also check the core
temperature of the FPGA in addition to VCCINT and VCCAUX, and tests the DONE
and INIT LEDS.
"""

vivado_settings = "/opt/Xilinx/Vivado_Lab/2021.1/settings64.sh"

pxct_image = [
"PXCT Board                          |",
"            (3 openings in the PCB) |",
"                         |          |",
"+-------------+          V          |",
"|             |                     |",
"|             |        /--\         |",
"|             |        |  | DONE LED|",
"|             |        \--/         |",
"|             |                     |",
"| (Opening to |        /--\         |",
"| PXIeCOMe    |        |  | INIT LED|",
"| module)     |        \--/         |",
"|             |                     |",
"|             |      /--\           |",
"|             |      |  | PROG      | Push the PROG button using",
"|             |      \--/ button    | a non-conductive rod or shaft.",
"|             |                     |",
"|             |                     | Observe the DONE and INIT LEDs.",
"+-------------+                     |",
"------------------------------------/"
]

def jtag_flash(arg="", default_directory='.'):
    mcs_file = "../../gateware/PXIeCOMe_tFPGA.mcs"
    cmd_settings = "source %s" % (vivado_settings)
    path_flash = os.path.join(default_directory, "./flash_v2")
    cmd_cd = "cd %s" % path_flash
    cmd_flash = "python ./flash.py -tool vivado_lab -mcs %s -flash s25fl128sxxxxxx1-spi-x1_x2_x4 %s" % (mcs_file, arg)
    cmd_cleanup = "rm *.backup.* -f"
    cmd = cmd_settings + "; " + cmd_cd + "; " + cmd_flash + "; " + cmd_cleanup
    try:
        df = subprocess.check_output(cmd, shell=True, stderr=subprocess.STDOUT)
        output = df.rstrip()
        status = 0
    except subprocess.CalledProcessError as e:
        output = e.output.rstrip()
        status = e.returncode
    print "%s" % (output)
    if status == 0:
        output_lines = output.split('\n')
        error_lines = [i for i in output_lines if i.startswith('ERROR:')]
        if len(error_lines) == 0:
            status = 0
        else:
            status = len(error_lines)

            stdio_redirect_start()
            for i in error_lines:
                print i
            stdio_redirect_end()
    else:
        status = -status
    return status

def jtag_flash_program(default_directory='.'):
    stdio_redirect_start()
    print ""
    print "Programming... (This takes about 90 seconds)."
    stdio_redirect_end()
    return jtag_flash("-erase 1 -blank_check 0 -program 1 -verify 1", default_directory)
    
def jtag_flash_erase(default_directory='.'):
    stdio_redirect_start()
    print ""
    print "Erasing... (This takes about 40 seconds)."
    stdio_redirect_end()
    return jtag_flash("-erase 1 -blank_check 0 -program 0 -verify 0", default_directory)

def vivado_tcl(tcl_file, default_directory='.'):
    cmd_settings = "source %s" % (vivado_settings)
    path_tcl = os.path.join(default_directory, "../tcl")
    cmd_cd = "cd %s" % path_tcl
    cmd_vivado = "vivado_lab -mode batch -nolog -nojournal -source %s" % (tcl_file)
    cmd_cleanup = "rm *.log -f; rm *.jou -f"
    cmd = cmd_settings + "; " + cmd_cd + "; " + cmd_vivado + "; " + cmd_cleanup
    value = "0.0"
    output = ""
    try:
        df = subprocess.check_output(cmd, shell=True, stderr=subprocess.STDOUT)
        output = df.rstrip()
        status = 0
    except subprocess.CalledProcessError as e:
        output = e.output.rstrip()
        status = e.returncode
    if status == 0:
        output_lines = output.split('\n')
        error_lines = [i for i in output_lines if i.startswith('ERROR:')]
        value_lines = [i for i in output_lines if i.startswith('VALUE:')]
        if len(error_lines) > 0:
            status = len(error_lines)
        else:
            if len(value_lines) != 0:
                value_line = value_lines[0]
                value = value_line[6:]
            status = 0
    else:
        status = -status
    return status, value, output

def vivado_temperature(default_directory='.'):
    stdio_redirect_start()
    print ""
    print "Reading FPGA core temperature..."
    stdio_redirect_end()
    return vivado_tcl("vivado_temperature.tcl", default_directory)

def vivado_vccint(default_directory='.'):
    stdio_redirect_start()
    print ""
    print "Reading FPGA VCCINT..."
    stdio_redirect_end()
    return vivado_tcl("vivado_vccint.tcl", default_directory)

def vivado_vccaux(default_directory='.'):
    stdio_redirect_start()
    print ""
    print "Reading FPGA VCCAUX..."
    stdio_redirect_end()
    return vivado_tcl("vivado_vccaux.tcl", default_directory)

def vivado_boot_device(default_directory='.'):
    stdio_redirect_start()
    print ""
    print "Booting FPGA..."
    stdio_redirect_end()
    return vivado_tcl("vivado_boot_device.tcl", default_directory)

tmp_stdout = sys.stdout
tmp_stdin = sys.stdin

def stdio_redirect_start():
    tmp_stdout = sys.stdout
    sys.stdout = sys.__stdout__
    tmp_stdin = sys.stdin
    sys.stdin = sys.__stdin__

def stdio_redirect_end():
    sys.stdout = tmp_stdout
    sys.stdin = tmp_stdin


def main (default_directory='.'):

    print "Test05 (JTAG) Start"
    init_test_time = time.time()
    ret_error = None
    abort = None
    status = 0
    output = ""
    print ""

    print "FPGA core temperature is ",
    status, value, output = vivado_temperature(default_directory)
    temperature = float(value)
    if status > 0:
        ret_error = 'Vivado Lab tools error.'
        print_error()
        print output
    else:
        if status < 0:
            ret_error = 'Shell command error: %d' % (-status)
            print_error()
            print output
        else:
            print "%3.1f deg C" % (temperature),
            if temperature > 45.0:
                ret_error = 'FPGA core temperature too high: %3.1f deg C' % (temperature)
                print_error()
            else:
                print_ok()
    print ""
    if (ret_error):
        raise PtsCritical(ret_error)
    
    print "FPGA VCCINT is",
    status, value, output = vivado_vccint(default_directory)
    vccint = float(value)
    if status > 0:
        ret_error = 'Vivado Lab tools error.'
        print_error()
        print output
    else:
        if status < 0:
            ret_error = 'Shell command error: %d' % (-status)
            print_error()
            print output
        else:
            print "%1.3f V" % (vccint),
            if vccint > 1.05 or vccint < 0.95:
                ret_error = 'FPGA VCCINT output range: %1.3f V' % (vccint)
                print_error()
            else:
                print_ok()
    print ""
    if (ret_error):
        raise PtsCritical(ret_error)

    print "FPGA VCCAUX is",
    status, value, output = vivado_vccaux(default_directory)
    vccaux = float(value)
    if status > 0:
        ret_error = 'Vivado Lab tools error.'
        print_error()
        print output
    else:
        if status < 0:
            ret_error = 'Shell command error: %d' % (-status)
            print_error()
            print output
        else:
            print "%1.3f V" % (vccaux),
            if vccaux > 1.85 or vccaux < 1.75:
                ret_error = 'FPGA VCCAUX output range: %1.3f V' % (vccaux)
                print_error()
            else:
                print_ok()
    print ""
    if (ret_error):
        raise PtsCritical(ret_error)

    status = jtag_flash_program(default_directory)
    print ""
    print "JTAG programming ",
    if status > 0:
        ret_error = 'Vivado Lab tools error.'
        print_error()
    else:
        if status < 0:
            ret_error = 'Shell command error: %d' % (-status)
            print_error()
        else:
            print_ok()
    print ""

    print "JTAG boot device",
    status, value, output = vivado_boot_device(default_directory)
    if status > 0:
        ret_error = 'Vivado Lab tools error.'
        print_error()
        print output
    else:
        if status < 0:
            ret_error = 'Shell command error: %d' % (-status)
            print_error()
            print output
        else:
            print_ok()
    print ""
    if (ret_error):
        raise PtsCritical(ret_error)

    stdio_redirect_start()
    print ("")
    for i in range(len(pxct_image)):
        print (" %s") % pxct_image[i]
    print ("")
    ask = "";
    while ((ask != "Y") and (ask != "N") and (ask != "A")):
        print "-------------------------------------------------------------"
        ask = raw_input("Does the DONE LED light up when the PROG button is pushed? [Y(es),N(o),A(bort)]")
        ask = ask.upper()
        print " "
    stdio_redirect_end()
    if (ask == "A"):
        abort = "Test aborted"
    else:
        print 'LED (DONE)',
        if (ask == "N"):
            ret_error = 'LED (DONE) not functioning.'
            print_error()
        if (ask == "Y"):
            print_ok()
    if abort:
        raise PtsCritical(abort)

    stdio_redirect_start()
    ask = "";
    while ((ask != "Y") and (ask != "N") and (ask != "A")):
        print "-------------------------------------------------------------"
        ask = raw_input("Did the INIT LED light up when the PROG button was pushed? [Y(es),N(o),A(bort)]")
        ask = ask.upper()
        print " "
    stdio_redirect_end()
    if (ask == "A"):
        abort = "Test aborted"
    else:
        print 'LED (INIT)',
        if (ask == "N"):
            ret_error = 'LED (INIT) not functioning.'
            print_error()
        if (ask == "Y"):
            print_ok()
    if abort:
        raise PtsCritical(abort)
    print ""

    end_test_time = time.time()
    print "End of Test05 (JTAG)"
    print "RESULT: [{}]".format("FAIL" if ret_error else "OK")
    print 'Test05 elapsed time: {:.2f} seconds'.format(end_test_time-init_test_time)
    if ret_error:
        raise PtsError(ret_error)
    

if __name__ == '__main__' :
    main()
