#!   /usr/bin/env   python
#    coding: utf8

# Copyright CERN, 2021-2022
# This source describes Open Hardware and is licensed under the
# CERN-OHL-W-2.0+
# Source location: https://ohwr.org/project/pxie-ctl-comexpress-tst
# designed by INCAA Computers
#------------------------------------------------------------------------
# Changes
# 2022-12-15 INCAA Computers
#   Removed GPIO_BASE_ADDR (base addr of the gpio is now set dynamically)
#   Added GPIOCHIP_LABEL
#   Added print of the used gpiochip
#------------------------------------------------------------------------

import sys
import time
import os
import subprocess
import string

# Add common modules and libraries location to path
sys.path.append('../../../')

from ptsexcept import *
from ptspxiecome import *

from COMe_GPIO import *

"""
test08: Check I2C and SMBus

Check the onboard I2C connection to the flash memory at 0x54.
And check the SMBus by reading the PXIe chassis ID eeprom at 0x52 or a device
at 0x2D.
The used PXIe chassis is a NI PXIe-1075, which has the backplane descriptor
eeprom at 0x52 and a backplane clocking CPLD at 0x2D

The I2C and SMBus interfaces are tested using one of the onboard i2c host
adapter devices. A list can be shown using the shell command 'i2cdetect -l'
For the I2C bus the host 'i2c-kempld' is used
For the SMBus the host 'SMiBus CMI adpater cmi' is used

On the Kontron COMe-bCL6 and COMe-BTL6 the SMBus is also used for the DDR
SO-DIMM SPD flash memory at 0x50 for slot 1 and at 0x52 for slot 2.
The PXIe chassis ID eeprom is also at 0x52 and could result in a conflict.

This conflict is however prevented by the COMe-bCL6 BIOS through the
'SMBus I801 adapter' host adapter. So the 'SMiBus CMI adpater cmi' can only
be used.
When a SO-DIMM SPD flash memory is present at 0x52 the conflict prevents the
access to the backplane descriptor eeprom to test the SMBus. To overcome this
the test adjusts to use the SMBus device at 0x2D.

NI PXIe-1075 devices:
    0x2D: backplane clocking CPLD
    0x2E: temperature monitoring device
    0x52: PXIe chassis ID eeprom / backplane descriptor eeprom

COMe-bCL6 devices
    0x2E: hardware monitor NCT7802Y  !!! Do not use under any circumstances !!!
    0x50: SPD DDR Channel 1 (SO-DIMM)
    0x52: SPD DDR Channel 2 (SO-DIMM)
    0x18: SO-DIMM Thermal Sensor channel 1 (If available on the used memory-module)
    0x1A: SO-DIMM Thermal Sensor channel 2 (If available on the used memory-module)

COMe-bTL6 devices
    0x2E: hardware monitor NCT7802Y  !!! Do not use under any circumstances !!!
    0x50: DDR4 channel A EEPROM
    0x52: DDR4 channel B EEPROM
    0x53: DDR4 channel C EEPROM (optional)
"""

I2C_ADAPTER = COMe_module_i2c_port() 
SMBUS_ADAPTER = COMe_module_smbus_port()

GPIOCHIP_LABEL = COMe_module_gpiochip_label()

ADDR_I2C_MEM_KONTRON = 0x50
ADDR_I2C_MEM_PXIeCOMe = 0x54
ADDR_SMBUS_MEM_CHASSIS = 0x52 # NI PXIe-1075 Backplane Descriptor EEPROM
ADDR_SMBUS_DEV_CHASSIS = 0x2D # NI PXIe-1075 Backplane Clocking CPLD

def smbus_get(bus, address, reg):
    # Use single access i2cget to overcome block read problems
    cmd = 'i2cget -y %d 0x%02x %d' % (bus, address, reg)
    try:
        df = subprocess.check_output(cmd, shell=True, stderr=subprocess.STDOUT)
        output = df.rstrip();
        status = 0
    except subprocess.CalledProcessError as e:
        output = e.output.rstrip()
        status = e.returncode
    if (status == 0):
        value = int(output, base=16)
    else:
        value = 0
    return value, status, output

def smbus_dump(bus, address):
    # Use single access i2cget to overcome block read problems
    dump = []
    for reg in range(0, 256):
        value, status, output = smbus_get(bus, address, reg)
        if (status == 0):
            dump.append(value)
        else:
            break
    return dump, status, output

def i2c_dump(bus, address):
    # use i2cdump in i2c mode to overcome stange behaviour of using multiple i2cget accesses
    dump = []
    cmd = 'i2cdump -y %d 0x%02x i' % (bus, address)
    try:
        df = subprocess.check_output(cmd, shell=True, stderr=subprocess.STDOUT)
        output = df.rstrip();
        status = 0
    except subprocess.CalledProcessError as e:
        output = e.output.rstrip()
        print output
        status = e.returncode
    if (status == 0):
        output_lines = output.split('\n')
        del output_lines[0]
        byte_list = []
        for line in output_lines:
            line = line.rstrip()
            line = line[4:51]
            byte_list.extend(line.split(' '))
        for b in byte_list:
            dump.append(int(b, base=16))
    return dump, status, output

def print_hex(byte_list):
    row_list = []
    cells = len(byte_list)
    print "     0  1  2  3  4  5  6  7  8  9  a  b  c  d  e  f    0123456789abcdef"
    row = 0
    for r in range(0, 16):
        if cells == 0:
            break
        if row_list:
            row_list[:] = []
        print "%02x:" % (row),
        for col in range(0, 16):
            if cells == 0:
                row_list.append(32)
                print "  ",
            else:
                byte = byte_list[row+col]
                row_list.append(byte)
                print "%02x" % (byte),
                cells -= 1
        print "  ",
        s = ""
        for b in row_list:
            ch = chr(b)
            if all(c in string.printable for c in ch):
                if b < 32:
                    s += '.'
                else:
                    s += ch
            else:
                s += '.'
        print "%s" % (s)
        row += 16
    
def find_str_in_list(byte_list, st):
    s = ""
    for b in byte_list:
        ch = chr(b)
        if all(c in string.printable for c in ch):
            if b < 32:
                s += '.'
            else:
                s += ch
        else:
            s += '.'
    return s.find(st)


def main (default_directory='.'):

    print "Test08 (I2C/SMBus) Start"
    init_test_time = time.time()
    ret_error = None

    # GPIO
    print ""
    come_gpio = COMe_GPIO(label=GPIOCHIP_LABEL)
    come_gpio.setup()
    print 'Using GPIO %s via gpiochip%d' % (GPIOCHIP_LABEL, come_gpio.baseaddr())
    print ""

    # I2C
    print "Access Kontron ID memory at I2C 0x%02x" % (ADDR_I2C_MEM_KONTRON)
    dump, status, output = i2c_dump(I2C_ADAPTER, ADDR_I2C_MEM_KONTRON)
    if status == 0:
        print_hex(dump)
        if find_str_in_list(dump, "Kontron") < 0:
            ret_error = "Kontron ID memory not found"
            print_error()
        else:
            print_ok()
    else:
        ret_error = "Kontron ID memory access error %d: %s" % (status, output)
        print_error()
    print ""
            
    print "Access PXIeCOMe memory at I2C 0x%02x" % (ADDR_I2C_MEM_PXIeCOMe)
    dump, status, output = i2c_dump(I2C_ADAPTER, ADDR_I2C_MEM_PXIeCOMe)
    if status == 0:
        print_hex(dump)
        print_ok()
    else:
        ret_error = "PXIeCOMe memory access error %d: %s" % (status, output)
        print_error()
    print ""
            
    # SMBus
    come_gpio.gpo(3, 0)

    checkSMBusMem = True
    print "Check access to memory at SMBus 0x%02x." % (ADDR_SMBUS_MEM_CHASSIS)
    dump, status, output = smbus_dump(SMBUS_ADAPTER, ADDR_SMBUS_MEM_CHASSIS)
    if status == 0:
        print_hex(dump)
        print "Memory found at 0x%02x. Probably conflict with %s device." % (ADDR_SMBUS_MEM_CHASSIS, COMe_module_name())
        print ""
        checkSMBusMem = False
        print "Check access to device at SMBus 0x%02x" % (ADDR_SMBUS_DEV_CHASSIS)
        value, status, output = smbus_get(SMBUS_ADAPTER, ADDR_SMBUS_DEV_CHASSIS, 0x00)
        if status == 0:
            print output
            ret_error = "Device found at 0x%02x. Probably conflict with %s device." % (ADDR_SMBUS_DEV_CHASSIS, COMe_module_name())
            print_error()
        else:
            if status == 2: # Read failed
                print "No device found at 0x%02x [OK]" % (ADDR_SMBUS_DEV_CHASSIS)
            else:
                ret_error = "SMBus device access error %d: %s." % (status, output)
                print_error()
    else:
        if status == 2: # Read failed
            print "No memory found at 0x%02x [OK]" % (ADDR_SMBUS_MEM_CHASSIS)
        else:
            ret_error = "SMBus memory access error %d: %s." % (status, output)
            print_error()
    print ""

    print "Enable SMBus access to chassis/backplane."
    come_gpio.gpo(3, 1)
    print ""

    if checkSMBusMem:
        print "Check access to memory at SMBus 0x%02x (should exist)." % (ADDR_SMBUS_MEM_CHASSIS)
        dump, status, output = smbus_dump(SMBUS_ADAPTER, ADDR_SMBUS_MEM_CHASSIS)
        if status == 0:
            print_hex(dump)
            print_ok()
        else:
            ret_error = "SMBus memory access error %d: %s." % (status, output)
            print_error()
    else:
        print "Check access to device at SMBus 0x%02x (should exist)." % (ADDR_SMBUS_DEV_CHASSIS)
        value, status, output = smbus_get(SMBUS_ADAPTER, ADDR_SMBUS_DEV_CHASSIS, 0x00)
        if status == 0:
            print_ok()
        else:
            ret_error = "SMBus device access error %d: %s." % (status, output)
            print_error()
    print ""

    come_gpio.gpo(3, 0)
    come_gpio.cleanup()

    print ""
    end_test_time = time.time()
    print "End of Test08 (I2C/SMBus)"
    print "RESULT: [{}]".format("FAIL" if ret_error else "OK")
    print 'Test08 elapsed time: {:.2f} seconds'.format(end_test_time-init_test_time)
    if ret_error:
        raise PtsError(ret_error)
    

if __name__ == '__main__' :
    main()
