#!   /usr/bin/env   python
#    coding: utf8

# Copyright CERN, 2021-2022
# This source describes Open Hardware and is licensed under the
# CERN-OHL-W-2.0+
# Source location: https://ohwr.org/project/pxie-ctl-comexpress-tst
# designed by INCAA Computers

import sys
import time
import os
import subprocess

# Add common modules and libraries location to path
sys.path.append('../../../')

from ptsexcept import *
from ptspxiecome import *

"""
test09: PCI Express

Check the interface to the PXIe backplane.
This is done by accessing the information regarding the PCI/PCI express devices in the system.
The PXIeCOMe provides four links of 4 lanes to the PXIe backplane. Every link should be connected
to a PCIe device like the PEX8525 and PEX8533 (see shell command 'lspci') on the backplane of
the NI PXIe-1075. So the link to these PEX8525 and PEX8533 devices should be 4 lanes wide and
provide a speed of at least 2.5 GT/s.
"""

def pci_link_exist(bus, slot=0, func=0):
    domain = 0
    sysfs_file = '/sys/bus/pci/devices/%04x:%02x:%02x.%01x' % (domain, bus, slot, func)
    return os.path.exists(sysfs_file)

def pci_link_is_upstream_port(bus, slot=0, func=0):
    domain = 0
    cmd = "lspci -s %x:%x:%x.%x -v" % (domain, bus, slot, func)
    try:
        df = subprocess.check_output(cmd, shell=True, stderr=subprocess.STDOUT)
        output = df.rstrip()
        status = 0
    except subprocess.CalledProcessError as e:
        output = e.output.rstrip()
        status = e.returncode
    if status == 0:
        loc = output.find("Upstream Port")
        if loc == -1:
            return False
        else:
            return True
    return False

def pci_link_attrib(file, bus, slot=0, func=0):
    domain = 0
    sysfs_file = '/sys/bus/pci/devices/%04x:%02x:%02x.%01x/%s' % (domain, bus, slot, func, file)
    return shell_cat_file(sysfs_file)

def pci_link_vendor(bus, slot=0, func=0):
    return pci_link_attrib("vendor", bus, slot, func)

def pci_link_device(bus, slot=0, func=0):
    return pci_link_attrib("device", bus, slot, func)

def pci_link_has_vendor_and_device(vendor, device, bus, slot=0, func=0):
    if ((pci_link_vendor(bus, slot, func) == vendor) and pci_link_device(bus, slot, func) == device):
        return True
    else:
        return False

def pci_link_speed(bus, slot=0, func=0):
    speed = pci_link_attrib("current_link_speed", bus, slot, func)
    if speed == "None":
        speed = "0 GT/s"
    loc = speed.find("GT/s")
    speed = speed[:loc]
    #speed = speed[:len(speed)-6]
    return float(speed)

def pci_link_width(bus, slot=0, func=0):
    width = pci_link_attrib("current_link_width", bus, slot, func)
    if width == "None":
        width = "0"
    return int(width)


def main (default_directory='.'):

    print "Test09 (PCI Express) Start"
    init_test_time = time.time()
    ret_error = None

    print ""
    for bus in range(0x00, 0x20):
        for slot in range(0x00, 0x20):
            if pci_link_exist(bus, slot):
                if (pci_link_has_vendor_and_device("0x10b5" , "0x8525", bus, slot) or
                            pci_link_has_vendor_and_device("0x10b5", "0x8533", bus, slot)):
                    if pci_link_is_upstream_port(bus, slot):
                        speed = pci_link_speed(bus, slot)
                        width = pci_link_width(bus, slot)
                        print "PCIe(%02x:%02x) speed %2.1f GT/s, width %d, id %s:%s" % (
                            bus, slot, speed, width,
                            pci_link_vendor(bus, slot),
                            pci_link_device(bus, slot),
                            ),
                        if width != 4:
                            ret_error = "PCIe(%02x:%02x) Incorrect width %2.1f" % (bus, slot, width)
                            print_error()
                        else:
                            if speed < 2.5:
                                ret_error = "PCIe(%02x:%02x) Incorrect speed %2.1f" % (bus, slot, speed)
                                print_error()
                            else:
                                print_ok()

    print ""
    end_test_time = time.time()
    print "End of Test09 (PCI Express)"
    print "RESULT: [{}]".format("FAIL" if ret_error else "OK")
    print 'Test09 elapsed time: {:.2f} seconds'.format(end_test_time-init_test_time)
    if ret_error:
        raise PtsError(ret_error)
    

if __name__ == '__main__' :
    main()
