#!   /usr/bin/env   python
#    coding: utf8

# Copyright CERN, 2021-2022
# This source describes Open Hardware and is licensed under the
# CERN-OHL-W-2.0+
# Source location: https://ohwr.org/project/pxie-ctl-comexpress-tst
# designed by INCAA Computers
#------------------------------------------------------------------------
# Changes
# 2022-11-14 INCAA Computers
#   Added COM0_DEVICE to PXCT()
#   Changed trig_level_write_test tolerance from 50 to 70
#   Changed trig_level_write_test tolerance from 25 to 35
#   Changed trig_level_read_test tolerance from 50 to 70
#   Changed trig_level_read_test tolerance from 25 to 35
#------------------------------------------------------------------------

import sys
import time
import os
import subprocess

# Add common modules and libraries location to path
sys.path.append('../../../')

from ptsexcept import *
from ptspxiecome import *

from PXIeCOMe_tFPGA import * 
from PXCT import * 

"""
test11: TRIG

Check the interface to the front panel TRIG port using one PXCT module.
"""

COM0_DEVICE = COMe_module_com0_device()

def trig_init(com_tFPGA, com_PXCT):
    # initialize all interface directions to input
    com_tFPGA.set_trig_direction()
    com_PXCT.set_trig_direction()
    # disable all terminations
    com_tFPGA.set_trig_termination(0)
    com_PXCT.set_trig_termination(0)
    # set default output value to low
    com_tFPGA.set_trig(0)
    com_PXCT.set_trig(0)
    
trig_write_test = [
    { 'write':0x0, 'read':0x0, 'term':0x0}, 
    { 'write':0x1, 'read':0x1, 'term':0x0}, 
    { 'write':0x0, 'read':0x0, 'term':0x0}, 
    { 'write':0x1, 'read':0x1, 'term':0x0}, 
    { 'write':0x0, 'read':0x0, 'term':0x1}, 
    { 'write':0x1, 'read':0x1, 'term':0x1}, 
    { 'write':0x0, 'read':0x0, 'term':0x1}, 
    { 'write':0x1, 'read':0x1, 'term':0x1} 
]

trig_read_test = [
    { 'write':0x0, 'read':0x0, 'term':0x0}, 
    { 'write':0x1, 'read':0x1, 'term':0x0},
    { 'write':0x0, 'read':0x0, 'term':0x0}, 
    { 'write':0x1, 'read':0x1, 'term':0x0},
    { 'write':0x0, 'read':0x0, 'term':0x1}, 
    { 'write':0x1, 'read':0x1, 'term':0x1},
    { 'write':0x0, 'read':0x0, 'term':0x1}, 
    { 'write':0x1, 'read':0x1, 'term':0x1}
]

def trig_test(com_tFPGA, com_PXCT):
    ret_error = None
    # TRIG write 
    trig_init(com_tFPGA, com_PXCT)
    for test in trig_write_test:
        print "TRIG test write 0x%X with termination %s" % (test['write'], 'On' if test['term'] else 'Off'),
        com_PXCT.set_trig_termination(test['term'])
        com_tFPGA.set_trig_direction(0)
        com_tFPGA.set_trig(test['write'])
        val = com_PXCT.get_trig()
        if val != test['read']:
            ret_error = "Incorrect TRIG value read: 0x%X, expected: 0x%X" % (val, test['read'])
            print "(read: 0x%X, expected: 0x%X)" % (val, test['read']),
            print_error()
        else:
            print_ok()
    print ""
    # TRIG read
    trig_init(com_tFPGA, com_PXCT)
    for test in trig_read_test:
        print "TRIG test read 0x%X with termination %s" % (test['write'], 'On' if test['term'] else 'Off'),
        com_tFPGA.set_trig_termination(test['term'])
        com_PXCT.set_trig_direction(0)
        com_PXCT.set_trig(test['write'])
        val = com_tFPGA.get_trig()
        if val != test['read']:
            ret_error = "Incorrect TRIG value read: 0x%X, expected: 0x%X" % (val, test['read'])
            print "(read: 0x%X, expected: 0x%X)" % (val, test['read']),
            print_error()
        else:
            print_ok()
    print ""
    trig_init(com_tFPGA, com_PXCT)
    return ret_error


trig_level_write_test = [
    { 'write':0x0, 'level':40, 'tolerance':10, 'term':0}, 
    { 'write':0x1, 'level':2850, 'tolerance':70, 'term':0}, 
    { 'write':0x0, 'level':15, 'tolerance':10, 'term':1}, 
    { 'write':0x1, 'level':1300, 'tolerance':35, 'term':1} 
]

trig_level_read_test = [
    { 'write':0x0, 'level':40, 'tolerance':10, 'term':0}, 
    { 'write':0x1, 'level':2850, 'tolerance':70, 'term':0}, 
    { 'write':0x0, 'level':15, 'tolerance':10, 'term':1}, 
    { 'write':0x1, 'level':1300, 'tolerance':35, 'term':1} 
]

def trig_level_test(com_tFPGA, com_PXCT):
    ret_error = None
    # TRIG write 
    trig_init(com_tFPGA, com_PXCT)
    for test in trig_level_write_test:
        print "TRIG level test write 0x%X with termination %s" % (test['write'], 'On' if test['term'] else 'Off'),
        com_PXCT.set_trig_termination(test['term'])
        com_tFPGA.set_trig_direction(0)
        com_tFPGA.set_trig(test['write'])
        level = com_PXCT.get_trig_level()
        if level > (test['level'] + test['tolerance']) or level < (test['level'] - test['tolerance']):
            ret_error = "Incorrect TRIG level read: %d, expected: %d +/- %d" % (level, test['level'], test['tolerance'])
            print ""
            print "\t(read: %d, expected: %d +/- %d)" % (level, test['level'], test['tolerance']),
            print_error()
        else:
            print_ok()
    print ""
    # TRIG read
    trig_init(com_tFPGA, com_PXCT)
    for test in trig_level_read_test:
        print "TRIG level test read 0x%X with termination %s" % (test['write'], 'On' if test['term'] else 'Off'),
        com_tFPGA.set_trig_termination(test['term'])
        com_PXCT.set_trig_direction(0)
        com_PXCT.set_trig(test['write'])
        level = com_PXCT.get_trig_level()
        if level > (test['level'] + test['tolerance']) or level < (test['level'] - test['tolerance']):
            ret_error = "Incorrect TRIG level read: %d, expected: %d +/- %d" % (level, test['level'], test['tolerance'])
            print ""
            print "\t(read: %d, expected: %d +/- %d)" % (level, test['level'], test['tolerance']),
            print_error()
        else:
            print_ok()
    print ""
    trig_init(com_tFPGA, com_PXCT)
    return ret_error


def main (default_directory='.'):

    print "Test11 (TRIG) Start"
    init_test_time = time.time()
    ret_error = None
    abort = None

    # Front panel COM port connected to the PXCT module in slot 2
    print ""
    com_PXCT_p = PXCT(device = COM0_DEVICE)
    com_PXCT_p.connect()
    response = com_PXCT_p.id()
    print 'PXCT (Peripheral Slot) ID response: %s' % (response),
    exp_response = 'CPE-PXCT-19916'
    if exp_response not in response:
        abort = 'Incorrect PXCT (Peripheral Slot) response.'
        print_error()
    else:
        print_ok()
    if abort:
        com_PXCT_p.close()
        raise PtsCritical(abort)

    # Internal serial port connected to the onboard FPGA
    print ""
    com_tFPGA = PXIeCOMe_tFPGA()
    com_tFPGA.connect()
    response = com_tFPGA.id()
    print 'FPGA ID response: %s' % (response)

    print ""
    
    # TRIG test
    ret = trig_test(com_tFPGA, com_PXCT_p)
    if ret != None:
        ret_error = ret
    
    # TRIG level test
    ret = trig_level_test(com_tFPGA, com_PXCT_p)
    if ret != None:
        ret_error = ret
        
    com_tFPGA.close()
    com_PXCT_p.close()

    print ""
    end_test_time = time.time()
    print "End of Test11 (TRIG)"
    print "RESULT: [{}]".format("FAIL" if ret_error else "OK")
    print 'Test11 elapsed time: {:.2f} seconds'.format(end_test_time-init_test_time)
    if ret_error:
        raise PtsError(ret_error)
    

if __name__ == '__main__' :
    main()
